"""
    Ecks plugin to collect process tree from a machine

   Copyright 2011 Chris Read (chris.read@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

def get_processes(parent, host, community):
    """ This is a plugin to be loaded by Ecks

    return an array of tuples containing (pid, proc_name, proc_args, status) for each process

    status is an int which maps to:
        running(1),
        runnable(2), -- waiting for resource (i.e., CPU, memory, IO)
        notRunnable(3), -- loaded but waiting for event
        invalid(4) -- not loaded

    """
    procs = (1,3,6,1,2,1,25,4,2,1) # HOST-RESOURCES-MIB
    data = parent.get_snmp_data(host, community, procs, 1)
    return map(parent._build_answer,
        parent._extract(data, int, 1), # PID
        parent._extract(data, str, 2), # Process Name
        parent._extract(data, str, 5), # Arguments
        parent._extract(data, int, 7), # Status
    )

    




"""
   Ecks plugin to collect system load average

   Copyright 2011 Chris Read (chris.read@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

from decimal import Decimal

def get_loadavg(parent, host, community):
    """ This is a plugin to be loaded by Ecks

    return an array of tuples containing (name, value) for each measured interval (normally 1, 5 and 15 minutes)

    name is the description of the interval returned by the host
    
    value is a python Decimal (see decimal module) containing the floating point representation of the load average
    """
    load = (1,3,6,1,4,1,2021,10,1) # UCD-SNMP-MIB
    data = parent.get_snmp_data(host, community, load, 1)
    return map(parent._build_answer,
        parent._extract(data, str, 2),
        map(Decimal, parent._extract(data, str, 3)),
    )



"""
    Ecks plugin to collect network interface information

   Copyright 2011 Chris Read (chris.read@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

def get_nic(parent, host, community):
    """ This is a plugin to be loaded by Ecks

    Info on the data returned is available at http://www.oidview.com/mibs/0/IF-MIB.html

    return an array of tuples containing (name, type, mtu, speed, mac, admin_status, oper_status,
        last_changed, in_bytes, in_u_pkts, in_nu_pkts, in_discard, in_err, in_unknown,
        out_bytes, out_u_pkts, out_nu_pkts, out_discard, out_err, out_queue) for each ethernet interface

    The type return value is an integer that can be looked up at http://www.iana.org/assignments/ianaiftype-mib

    Most common type is 6 (ethernet)

    """
    nics = (1,3,6,1,2,1,2,2,1) # IF-MIB
    data = parent.get_snmp_data(host, community, nics, 1)

    return map(parent._build_answer,
        parent._extract(data, str, 2), # Name
        parent._extract(data, int, 3), # ifType
        parent._extract(data, int, 4), # MTU
        parent._extract(data, long, 5), # Speed
        parent._extract(data, str, 6), # MAC
        parent._extract(data, int, 7), # Admin Status
        parent._extract(data, int, 8), # Oper Status
        parent._extract(data, int, 9), # Last Change
        parent._extract(data, long, 10), # In Bytes
        parent._extract(data, long, 11), # In Unicast Pkts
        parent._extract(data, long, 12), # In Not Unicast Pkts
        parent._extract(data, long, 13), # In Discards 
        parent._extract(data, long, 14), # In Errors
        parent._extract(data, long, 15), # In Unknown Protocols
        parent._extract(data, long, 16), # Out Bytes
        parent._extract(data, long, 17), # Out Unicast Pkts
        parent._extract(data, long, 18), # Out Not Unicast Pkts
        parent._extract(data, long, 19), # Out Discards
        parent._extract(data, long, 20), # Out Errors
        parent._extract(data, long, 21), # Out Queue Length
    )



"""
    Ecks plugin to collect disk usage information

   Copyright 2011 Chris Read (chris.read@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

from pprint import pprint

def _calc_size(block_size, block_count):
    if block_size and block_count:
        return block_size * block_count
    else:
        return -1


def get_disk(parent, host, community):
    """ This is a plugin to be loaded by Ecks

    return an array of tuples containing (type, path, size in bytes, used bytes) for each block device

    type is an integer which is one of the following:
        hrStorageOther	        = 1
        hrStorageRam	        = 2
        hrStorageVirtualMemory	= 3
        hrStorageFixedDisk	    = 4
        hrStorageRemovableDisk	= 5
        hrStorageFloppyDisk	    = 6
        hrStorageCompactDisc	= 7
        hrStorageRamDisk	    = 8
        hrStorageFlashMemory	= 9
        hrStorageNetworkDisk	= 10
    """
    disks = (1,3,6,1,2,1,25,2,3,1) # HOST-RESOURCES-MIB
    data = parent.get_snmp_data(host, community, disks, 1)

    # We need to work this out the long was as there are cases where size or used is not supplied
    details = []
    formatted = []

    for i in [t for t in parent._extract(data, int, 1)]:
      details.append([ value for (oid, (data_type, index), value) in data if index == i  and data_type != 1])

    for dev in details:
      if len(dev) != 5:
        continue
      formatted.append((
        tuple(dev[0])[-1],
        str(dev[1]),
        int(dev[2]) * int(dev[3]),
        int(dev[2]) * int(dev[4])
        ))

    return formatted

